#  Object Name   : $RCSfile: imagetool.tcl,v $
#  Revision      : $Revision: 1.1.1.1 $
#  Date          : $Date: 2004/01/28 14:31:30 $
#  Author        : $Author: dgroth $
#  $Log: imagetool.tcl,v $
#  Revision 1.1.1.1  2004/01/28 14:31:30  dgroth
#  initial import into cvs
#
#
##############################################################################

snit::type imagetool {
    # Richard Suchenwirth
    method scaleImage {im xfactor {yfactor 0}} {
        set mode -subsample
        if {abs($xfactor) < 1} {
            set xfactor [expr round(1./$xfactor)]
        } elseif {$xfactor>=0 && $yfactor>=0} {
            set mode -zoom
        }
        if {$yfactor == 0} {set yfactor $xfactor}
        set t [image create photo]
        $t copy $im
        $im blank
        if {$xfactor >= 1} {
            set xfactor [string range $xfactor 0 0]
        }
        if {$yfactor >= 1} {
            set yfactor [string range $yfactor 0 0]
        }
        $im copy $t -shrink $mode $xfactor $yfactor
        image delete $t
    }
    method canvas2Photo { canvId } {
        # The following line grabs the contents of the canvas canvId into photo image ph.
        set retVal [catch {image create photo -format window -data $canvId} ph]
        if { $retVal != 0 } {
            puts "\n\tFATAL ERROR: Cannot create photo from canvas window"
            exit 1
        }
        return $ph
    }
    
    method rotateimg {ph angle} {
        #start of rotate code, ph is a handle to text/image written to a canvas
        #use canvas2Photo to get this handle
        set normdata [split [string map {, "" \} "" \" "" ; ""} [$ph data  -format XPM]] "\n"]
        proc lreverse {list} {
            set res {}
            foreach i $list {set res [concat [list $i] $res]}
            set res
        }
        #puts $normdata
        set rotdata "[string map {\{ "" \} "" \\ ""} [lindex $normdata 0]]\n"
        append rotdata "[string map {\{ "" \} "" \\ ""} [lindex $normdata 1]]\{\n"
        set dims [lindex $normdata 2]
        set width [lindex $dims 0]
        set height [lindex $dims 1]
        set colors [lindex $dims 2]
        set misc [lindex $dims 3]
        
        set bitimage [lrange $normdata [expr 3 + {$colors}] end]
        set currow 0
        array set rotrow {0 ""}
        array set bitarray {0 ""}
        switch $angle {
            270 {
                set colstart [expr {$width} - 1]
                set coltest -1
                set colincr -1
                append rotdata "\"$height $width $colors $misc\""
            }
            
            90 {
                set colstart 0
                set coltest $width
                set colincr 1
                set bitimage [lreverse $bitimage]
                append rotdata "\"$height $width $colors $misc\""
            }
            
            45 {
                set angsize [expr {$width  + $height - 1} ]
                append rotdata "\"$angsize $angsize $colors $misc\""
                set colstart 0
                set coltest $width
                set colincr 1
            }
            
            225 {
                set angsize [expr {$width  + $height - 1} ]
                append rotdata "\"$angsize $angsize $colors $misc\""
                set colstart 0
                set coltest $width
                set colincr 1
            }
            
        }
        
        for {set row 3} {$row < [expr {$colors} + 3]} {incr row} {
            append rotdata ",\n\"[lindex $normdata $row]\""
        }
        
        switch $angle {
            90  -
            
            270 { for {set col $colstart} {$col != $coltest} {incr col $colincr} {
                    foreach rowdata $bitimage {
                        append rotrow($currow) [string index $rowdata $col]
                    }
                    append rotdata ",\n\"$rotrow($currow)\""
                    incr currow
                }
            }
            45 { foreach rowdata $bitimage {
                    for {set col 0} {$col < $width} {incr col 1} {
                        set x2col [expr {$col} + ({$height}  - {$currow} - 1 )]
                        set row [expr {$col + $currow} ]
                        set bitarray($row,$x2col) [string index $rowdata  $col]
                    }
                    incr currow +1
                }
                
                set last "a"
                for {set row 0} {$row < $angsize } {incr row +1} {
                    append rotdata ",\n\""
                    for {set col 0} {$col < $angsize} {incr col +1} {
                        if {! [info exists bitarray($row,$col)]} {
                            append rotdata $last
                        } else {
                            set last $bitarray($row,$col)
                            append rotdata $bitarray($row,$col)
                        }
                    }
                    append rotdata "\""
                }
            }
            
            225 { foreach rowdata $bitimage {
                    for {set col 0} {$col < $width} {incr col 1} {
                        set row [expr {$col} + ({$height}  - {$currow} - 1 )]
                        set x2col [expr {$col + $currow} ]
                        set bitarray($row,$x2col) [string index $rowdata  $col]
                    }
                    incr currow +1
                }
                
                set last "a"
                for {set row $angsize} {$row > 0 } {incr row -1} {
                    append rotdata ",\n\""
                    for {set col 0} {$col < $angsize} {incr col +1} {
                        if {! [info exists bitarray($row,$col)]} {
                            append rotdata $last
                        } else {
                            set last $bitarray($row,$col)
                            append rotdata $bitarray($row,$col)
                        }
                    }
                    append rotdata "\""
                }
            }
            
        }
        append rotdata "\};"
        set rh [image create photo -data $rotdata]
        return $rh
        #end of rotate code, rh is a handle to the rotated version of the image
    }
    method rotate {img angle} {
        if $angle {
            set w [image width  $img]
            set h [image height $img]
            set tmp [image create photo]
            $tmp copy $img
            $img blank
            set buf {}
            switch -- $angle {
                90 {
                    for {set i [expr {$w-1}]} {$i>=0} {incr i -1} {
                        set rowbuf {}
                        for {set j 0} {$j < $h} {incr j} {
                            foreach {r g b} [$tmp get $i $j] break
                            lappend rowbuf [format #%02x%02x%02x $r $g $b]
                        }
                        lappend buf $rowbuf
                    }
                    $img config -width $h -height $w
                    $img put $buf
                }
                180 - -180 {$img copy $tmp -subsample -1 -1}
                270 - -90 {
                    for {set i 0} {$i<$w} {incr i} {
                        set rowbuf {}
                        for {set j [expr {$h-1}]} {$j>=0} {incr j -1} {
                            foreach {r g b} [$tmp get $i $j] break
                            lappend rowbuf [format #%02x%02x%02x $r $g $b]
                    }
                    lappend buf $rowbuf
                }
                $img config -width $h -height $w
                $img put $buf
            }
            default {
                set a [expr {atan(1)*8*$angle/360.}]
                set xm  [expr {$w/2.}]
                set ym  [expr {$h/2.}]
                set w2  [expr {round(abs($w*cos($a)) + abs($h*sin($a)))}]
                set xm2 [expr {$w2/2.}]
                set h2  [expr {round(abs($h*cos($a)) + abs($w*sin($a)))}]
                set ym2 [expr {$h2/2.}]
                $img config -width $w2 -height $h2
                for {set i 0} {$i<$h2} {incr i} {
                    set toX -1
                    for {set j 0} {$j<$w2} {incr j} {
                        set rad [expr {hypot($ym2-$i,$xm2-$j)}]
                        set th  [expr {atan2($ym2-$i,$xm2-$j)+$a}]
                        set x [expr {$xm-$rad*cos($th)}]
                        if {$x < 0 || $x >= $w} continue
                        set y [expr {$ym-$rad*sin($th)}]
                        if {$y < 0 || $y >= $h} continue
                        set x0 [expr {int($x)}]
                        set x1 [expr {($x0+1)<$w? $x0+1: $x0}]
                        set dx [expr {$x1-$x}]
                        set y0 [expr {int($y)}]
                        set y1 [expr {($y0+1)<$h? $y0+1: $y0}]
                        set dy [expr {$y1-$y}]
                        set R 0; set G 0; set B 0
                        foreach {r g b} [$tmp get $x0 $y0] {
                            set R [expr {$R+$r*$dx*$dy}]
                            set G [expr {$G+$g*$dx*$dy}]
                            set B [expr {$B+$b*$dx*$dy}]
                        }
                        foreach {r g b} [$tmp get $x0 $y1] {
                            set R [expr {$R+$r*$dx*(1.-$dy)}]
                            set G [expr {$G+$g*$dx*(1.-$dy)}]
                            set B [expr {$B+$b*$dx*(1.-$dy)}]
                        }
                        foreach {r g b} [$tmp get $x1 $y0] {
                            set R [expr {$R+$r*(1.-$dx)*$dy}]
                            set G [expr {$G+$g*(1.-$dx)*$dy}]
                            set B [expr {$B+$b*(1.-$dx)*$dy}]
                        }
                        foreach {r g b} [$tmp get $x1 $y1] {
                            set R [expr {$R+$r*(1.-$dx)*(1.-$dy)}]
                            set G [expr {$G+$g*(1.-$dx)*(1.-$dy)}]
                            set B [expr {$B+$b*(1.-$dx)*(1.-$dy)}]
                        }
                        set r [expr {round($R)}]
                        set g [expr {round($G)}]
                        set b [expr {round($B)}]
                        lappend buf [format #%02x%02x%02x $r $g $b]
                        if {$toX == -1} {set toX $j}
                    }
                    if {$toX>=0} {
                        $img put [list $buf] -to $toX $i
                        set buf {}
                        if $::update update
                    }
                }
            }
        }
        image delete $tmp
    }
   }
   # ulis
   method shrink {Image coef} {
        # check coef
        if {$coef > 1.0} \
                  { error "bad coef \"$coef\": should not be greater than 1.0" }
        if {abs($coef - 1.0) < 1.e-4} { return $Image }
        # get the old image content
        set Width [image width $Image]
        set Height [image height $Image]
        if {$Width * $Height == 0} { error "bad image" }
        # create corresponding planes
        for {set Y 0} {$Y < $Height} {incr Y} {
            set r:Row {}
            set g:Row {}
            set b:Row {}
            for {set X 0} {$X < $Width} {incr X} {
                foreach {r g b} [$Image get $X $Y] break
                foreach c {r g b} { lappend $c:Row [set $c] }
            }
            foreach c {r g b} { lappend $c:Data [set $c:Row] }
        }
        # compute the new image content
        set width [expr {round($Width * $coef)}]
        set height [expr {round($Height * $coef)}]
        set ey 0
        set Y2 0
        set cy2 $height
        for {set y 0} {$y < $height} {incr y} {
            set r:row {}
            set g:row {}
            set b:row {}
            # Y1 is the top coordinate in the old image
            set Y1 $Y2
            set cy1 [expr {$height - $cy2}]
            incr ey $Height
            set Y2 [expr {$ey / $height}]
            set cy2 [expr {$ey % $height}]
            if {$Y1 == $Y2} { set cy1 $cy2 }
            set ex 0
            set X2 0
            set cx2 $width
            for {set x 0} {$x < $width} {incr x} {
                set X1 $X2
                set cx1 [expr {$width - $cx2}]
                incr ex $Width
                set X2 [expr {$ex / $width}]
                set cx2 [expr {$ex % $width}]
                if {$X1 == $X2} { set cx1 $cx2 }
                # compute pixel
                foreach c {r g b} { set $c 0; set _$c 0 }
                for {set Y $Y1} {$Y <= $Y2} {incr Y} {
                    # compute y coef
                    switch $Y \
                              $Y1     { set cy $cy1 } \
                              $Y2     { set cy $cy2 } \
                              default { set cy $height }
                    if {$cy == 0} { continue }
                    if {$cy > $Height} { set cy $Height }
                    for {set X $X1} {$X <= $X2} {incr X}  {
                        # compute x coef
                        switch $X \
                                  $X1     { set cx $cx1 } \
                                  $X2     { set cx $cx2 } \
                                  default { set cx $width }
                        if {$cx == 0} { continue }
                        if {$cx > $Width} { set cx $Width }
                        # weight each initial pixel by cx & cy
                        set cxy [expr {$cx * $cy / double($Width) / $Height}]
                        foreach c {r g b} \
                                  {
                            set comp [lindex [set $c:Data] $Y $X]
                            incr $c [expr {round($comp * $cxy)}]
                            set _$c [expr {[set _$c] + $cxy}]
                        }
                    }
                }
                set _ {}
                foreach c {r g b} \
                          {
                    set comp [set $c]
                    if {$comp > 255} { set comp 255 }
                    lappend $c:row $comp
                    lappend _ [set _$c]
                }
            }
            foreach c {r g b} { lappend $c:data [set $c:row] }
        }
        # merge the planes
        for {set y 0} {$y < $height} {incr y} \
                  {
            set row {}
            for {set x 0} {$x < $width} {incr x} \
                      {
                foreach c {r g b} { set $c [lindex [set $c:data] $y $x] }
                lappend row [format #%02x%02x%02x $r $g $b]
                             }
                             lappend data $row
                         }
                         # create the new image
                         set image [image create photo]
                         # fill the new image
                         $image put $data
                         # return the new image
                         return $image
                     }
}
