#############################################################################
#
#  			Copyright 2003 MPIMG Berlin, Germany.
#			      All Rights Reserved
#  Created By    : Dr. Detlef Groth nocatdgroth@molgen.mpg.de, nocatdetlef@dgroth.de
#                    delete nocat to answer
#  Created       : Mon Mar 10 16:41:10 2003
#  Last Modified : <040114.1038>
#
#  Description	A simple htmlwidget providing the methods
#               load:  popups a dialog for local file selection
#               browse: loads either a local file or a webpage
#               back, forward, home, Refresh: just like a normal browser behaves
#               popup : installs a popup with access to back, forward, home, refresh
#  SYNOPSIS     
#              pack [snitbrowser .browser] -side left -fill both -expand yes
#              # WEB
#             .browser browse "http://mini.net/tcl/2993"
#             # LOCALFILES
#             .browser load 
#             # installing the popup
#             .browser popup .popup
#  Notes
#
#  History:  V0.1 13/11/03: First Release a three hours hack
#            V0.2 14/11/03: Code Refactoring:
#                           - Following William H. Duquette suggestions naming public methods
#                             lowercase
#                           - implementing public variables as options, like -showimages
#                           - resorting loading of packages
#                           - renaming LoadFile to the more appropiate `browse method
#                           - removing lonly global var popupvar
#  Bugs:     - I would like to change some coderefs from self to mymethod as 
#              suggested by William but this fails (try inside method home)
#  $Log$
#
##############################################################################
#    License:
#    Copyright (c) Dr. Detlef Groth, 2003 MPIMG Berlin, Germany.
#
#    The authors hereby grant permission to use, copy, modify, distribute,
#    and license this software and its documentation for any purpose,
#    provided that existing copyright notices are retained in all copies and
#    that this notice is included verbatim in any distributions. No written
#    agreement, license, or royalty fee is required for any of the
#    authorized uses.  Modifications to this software may be copyrighted by
#    their authors and need not follow the licensing terms described here,
#    provided that the new terms are clearly indicated on the first page of
#    each file where they apply.
#    
#    IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
#    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
#    ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
#    DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#    
#    THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
#    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
#    FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
#    IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
#    NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
#    MODIFICATIONS.
#    
#    GOVERNMENT USE: If you are acquiring this software on behalf of the
#    U.S. government, the Government shall have only "Restricted Rights"
#    in the software and related documentation as defined in the Federal 
#    Acquisition Regulations (FARs) in Clause 52.227.19 (c) (2).  If you
#    are acquiring the software on behalf of the Department of Defense, the
#    software shall be classified as "Commercial Computer Software" and the
#    Government shall have only "Restricted Rights" as defined in Clause
#    252.227-7013 (c) (1) of DFARs.  Notwithstanding the foregoing, the
#    authors grant the U.S. Government and others acting in its behalf
#    permission to use and distribute the software in accordance with the
#    terms specified in this license.
################################################################################
package provide snitbrowser 0.2
package require Tk
package require http
lappend auto_path [file join [file dirname [info script]] ..]
package require BWidget


package require snit 0.91
package require Tkhtml 
package require Wikit
snit::widget snitbrowser {
    # methods starting with underlines/Uppercases are private 
    # and should not be used externally
    variable htmlwidget
    variable lastDir
    variable Images 
    variable OldImages 
    variable hotkey
    variable LastFile
    variable PrevFiles 
    variable NextFiles
    variable Priv
    variable Popupvar
    variable popup
    option -showimages 1
    option -padx 5
    option -pady 5
    option -underlinehyperlinks 0
    option -bg white
    option -tablerelief raised
    option -home ""
    option -fontsize 12
    option -browsecmd 
    delegate option * to htmlwidget
    delegate method * to htmlwidget
    typeconstructor {
      image create photo smgray -data {
         R0lGODdhOAAYAPAAALi4uAAAACwAAAAAOAAYAAACI4SPqcvtD6OctNqLs968+w+G4kiW5omm
         6sq27gvH8kzX9m0VADv/
      }
    }   
    constructor {args} {
        set lastDir [pwd]
        set LastFile {}
        set PrevFiles {}
        set NextFiles {}
        pack [frame $win.f] -side top -fill both -expand yes
        html $win.f.html \
                  -yscrollcommand "$win.f.vsb set" \
                  -xscrollcommand "$win.hsb set" \
                  -imagecommand [mymethod _ImageCmd] \
                  -formcommand [mymethod _FormCmd] \
                  -fontcommand [mymethod _FontCmd] \
                  -unvisitedcolor blue \
                  -bg $options(-bg) \
                  -underlinehyperlinks $options(-underlinehyperlinks) \
                  -pady $options(-pady) -padx $options(-padx) \
                  -tablerelief $options(-tablerelief)
        pack $win.f.html -side left -fill both -expand 1
        scrollbar $win.f.vsb -orient vertical -command "$win.f.html yview"
        pack $win.f.vsb -side right -fill y
        bind $win.f.html.x <1> [mymethod _HrefBinding %x %y]
        bind $win.f.html.x <B1-Motion> [mymethod _SelectionBinding %W %x %y]
        bind $win.f.html.x <KeyPress-space> [list $win.f.html yview scroll +1 pages]
        bind $win.f.html.x <KeyPress-Next> [list $win.f.html yview scroll +1 pages]
        bind $win.f.html.x <Shift-space> [list $win.f.html yview scroll -1 pages]
        bind $win.f.html.x <KeyPress-Prior> [list $win.f.html yview scroll -1 pages]
        bind $win.f.html.x <KeyPress-Up> [list $win.f.html yview scroll -2 units]
        bind $win.f.html.x <KeyPress-Down> [list $win.f.html yview scroll -2 units]
        bind $win.f.html.x <Control-Home> [list $win.f.html yview moveto 0.0] 
        bind $win.f.html.x <Control-End> [list $win.f.html yview moveto 1.0] 
        scrollbar $win.hsb -orient horizontal -command "$win.f.html xview"
        pack $win.hsb -side bottom -fill x
        set htmlwidget $win.f.html
        image create photo smgray -data {
            R0lGODdhOAAYAPAAALi4uAAAACwAAAAAOAAYAAACI4SPqcvtD6OctNqLs968+w+G4kiW5omm
            6sq27gvH8kzX9m0VADv/
        }
        bind HtmlClip <Motion> {
            set parent [winfo parent %W]
            set url [$parent href %x %y] 
            if {[string length $url] > 0} {
                $parent configure -cursor hand2
            } else {
                $parent configure -cursor {}
            }
        }
        $self configurelist $args
    }
    onconfigure -fontsize {value} {
        set options(-fontsize) $value
        $self refresh
    }
    method home {} {
        if {$options(-home) ne ""} {
            $self browse $options(-home)  
        } else {
            $self browse [lindex $PrevFiles 0]  
        }
    }
    method back {} {
        $self browse [lindex $PrevFiles end]  
    }
    method forward {} {
        $self browse [lindex $NextFiles end]  
    }
    method geturl {} {
        return $LastFile
    }
    method search {text case forward} {
        #puts "forward: $forward"
        set cmd [list $htmlwidget text find $text]
        if {!$case} {
            lappend cmd nocase
        }
        set last [$htmlwidget index sel.last]
        #puts "last : $last"
        if {$forward} {
            set dir after
        } else {
            set dir before
        }
        if {$last != ""} {
            lappend cmd $dir $last
        } else {
            lappend cmd $dir
        }
        #        puts "cmd $cmd"
        set found [eval $cmd]
        if {[llength $found] == 0} {
            $htmlwidget selection clear
            tk_messageBox -title "Info!" -icon info -message "No (more) match: $text !" -type ok
        } else {
            #puts "found $found"
            foreach {begin end} $found break
            # Since text find returns shorter region, we should incr $end
            $htmlwidget selection set $begin [$self _Incr_html_index $end]
            $self _See $htmlwidget.x $begin $text $case $forward
        }
    }
    method _See {w ix text case forward} {
      
        foreach {x y} [$w coords $ix] break
        if {[info exists y]} {
            foreach {mx my} [$w coords] break
            $w yview moveto [expr {1.0*$y/$my}]
            $w xview moveto [expr {1.0*$x/$mx}]
        } else {
            # to fix a bug for first search because there y does not exists
            $self search $text $case $forward
        }
    }
    method _Incr_html_index {ix {off 1}} {
        foreach {token char} [split $ix .] break
        return $token.[incr char $off]
    } 
    method refresh {args} {
        if {![info exists LastFile]} return
        set yv [$htmlwidget yview]
        $self browse $LastFile
        # todo
        # this does not work
        $htmlwidget.x yview moveto [lindex $yv 0]
    }
    method _Popup {} {
        if {[winfo exists $Popupvar] } {
            set x [winfo pointerx .]
            set y [winfo pointery .]
            tk_popup $Popupvar $x $y 
        }
    }
   method browse {name} {
        # dgroth fix
       
        if {$name eq ""} { return }
      
        #puts "browsecmd: $options(-browsecmd)"
        if {$options(-browsecmd) ne ""} {
                 $options(-browsecmd)
         }
        # jcw 06/10/2000 - drop "#tag", if present
        set basename [lindex [split $name #] 0]
                                              
        set htmltxt [$self _ReadFile $basename]
                            
        if {[regexp -nocase {<noframe>.*<a href= *"?(.+?)"? *>.+</a></noframe} [string range $htmltxt 0 400] -> url]} {
            # in case of frames grab the link of the noframe region
            regexp {(.+/)} $name -> prefix
            $self browse "$prefix$url"
            return
        } elseif {[regexp -nocase {<frame src= *"?([^\s]+?)"? name} [string range $htmltxt 0 400] -> url]} {
            # or grab the first src which is mostly the content frame
            regexp {(.+/)} $name -> prefix
            $self browse "$prefix$url"
            return
        } 
    
        if {$htmltxt==""} return  
        #puts "browse: $name"  
        $self _Clear
        $self _History $name
        $htmlwidget config -base $name
     
        # jcw 06/10/2000 - deal with text files (as suggested by Uwe Koloska)
        if {![regexp -nocase {<html>|<!doctype|<body} [string range $htmltxt 0 200]]} {
            set htmltxt "<pre>$htmltxt</pre>\n"
        }
        # jcw: end of changed code
        $htmlwidget parse $htmltxt
        $self _ClearOldImages
       # dgroth 13/11/2003 add jumping to internal name
       if {[regexp {(.+)#(.+)} $LastFile match file anchor]} {
            #tk_messageBox -title "Info!" -icon info -message "snitbrowser2 Anchor: Name: $name Lastfile: $LastFile ..." -type ok
            
            #tk_messageBox -title "Info!" -icon info -message "message jumping to $anchor" -type ok
            update ;# fixing tha anchor bug
            $htmlwidget yview $anchor
           
         }
         if {$options(-browsecmd) ne ""} {
                 $options(-browsecmd)
         }
                              
    }
    method bind {event args} {
        eval {bind $htmlwidget.x $event} $args
    }
    method popup {path} {
        set Popupvar $path
        menu $Popupvar -tearoff 0
        $Popupvar add command -label Back -underline 1 -accelerator Alt+Left \
                  -command [mymethod back]
        $Popupvar add command -label Forward -underline 1 -accelerator Alt+Right \
                  -command [mymethod forward]
        $Popupvar add command -label Home     -underline 1 -accelerator Alt+Home  \
                  -command [mymethod home]
        $Popupvar add separator  
        $Popupvar add command -label Refresh -underline 0 -accelerator Ctrl-r \
                  -command [mymethod refresh]
       
     
        bind $htmlwidget.x <ButtonPress-3> [mymethod _Popup]
        bind all <Alt-Left> [ mymethod back ]
        bind all <Alt-Right> [ mymethod forward ]
        bind all <Alt-Home> [mymethod home]
    }
    method _MoveBigImage b {
      if {![info exists BigImages($b)]} return
      $b copy $BigImages($b)
      image delete $BigImages($b)
      unset BigImages($b)
      update
    }

    method _FetchImage {src w h args} {
        # Fetch the image
        if {[catch {
             http::geturl $src -timeout 10000
        } token]} {
             return smgray
        }
        set data [http::data $token]
        http::cleanup $token
        # Hack needed to make sure the data is binary:
        binary scan $data {}
        set name [image create photo]
        if {[catch {$name put $data} ret]} {
            return smgray
        }
        return $name
    }
    # anyone interested in implementing the _FormCmd ?
    method _FrameCmd {args} {
        # seems to be not invoked. What is TDB
        puts "framecmd: $args"
    }
    method _FormCmd {args} {
        puts "anyone interested in implementing the _FormCmd "
        #puts $args      
    }
    method _FontCmd {size attrs} { 
        #        puts "actual size: $options(-fontsize)"
        #       puts "FontCmd: $size $attrs"
        set a [expr {-1<[lsearch $attrs fixed]?{courier}:{charter}}]
        set b [expr {-1<[lsearch $attrs italic]?{italic}:{roman}}]
        set c [expr {-1<[lsearch $attrs bold]?{bold}:{normal}}]
        set d [expr {int($options(-fontsize)*pow(1.2,$size-4))}]
        list $a $d $b $c
   } 
    method _ImageCmd {args} {
        if {!$options(-showimages)} {
            return smgray
        }
        set fn [lindex $args 0]
        if {[string match {http:*} $fn]} {
            set dat [$self _FetchImage $fn 1 1]
            return $dat
        }
        if {[info exists OldImages($fn)]} {
            set Images($fn) $OldImages($fn)
            unset OldImages($fn)
            return $Images($fn)
        }
     
        if {[catch {image create photo -file $fn} img]} {
            return smgray
        }
       
        if {[image width $img]*[image height $img]>200000} {
            set b [image create photo -width [image width $img] \
                   -height [image height $img]]
            set BigImages($b) $img
            set img $b
            after idle "$self _MoveBigImage $b"
        }
        set Images($fn) $img
        return $img
    }
   
    method _SelectionBinding {w x y} {
        $htmlwidget selection set @$Priv(mark) @$x,$y
        clipboard clear
        # avoid tkhtml0.0 errors 
        # anyone can fix this for tkhtml0.0
        catch {
            clipboard append [selection get]
        }
    }
    method _History {name} {
      if {$name != $LastFile && $LastFile != ""} {
            if {$name == [lindex $PrevFiles end]} {
                set NextFiles [linsert $NextFiles 0 $LastFile]
                set PrevFiles [lreplace $PrevFiles end end]
            } else {
                lappend PrevFiles $LastFile
                
                if {$name == [lindex $NextFiles 0]} {
                    set NextFiles [lrange $NextFiles 1 end]
                } else {
                    set NextFiles {}
                }
            }
        }
        set LastFile $name
    }       
    method _HrefBinding {x y} {
        set Priv(mark) $x,$y
        set list [$htmlwidget href $x $y]
        if {![llength $list]} {return}
        foreach {new target} $list break
        if {$new!=""} {
            set pattern $LastFile#
            set len [string length $pattern]
            incr len -1
            if {[string range $new 0 $len]==$pattern} {
                incr len
                $htmlwidget yview [string range $new $len end]
                if {$options(-browsecmd) ne ""} {
                    $options(-browsecmd)
                }
                 $self _History $new 
            } else {
                $self browse $new
            }
        }
    }
    method _Clear {} {
        $htmlwidget clear
        catch {unset hotkey}
        $self _ClearBigImages
        $self _ClearOldImages
        foreach fn [array names Images] {
            set OldImages($fn) $Images($fn)
        }
        catch {unset Images}
    }
    method _ClearOldImages {} {
        foreach fn [array names OldImages] {
            image delete $OldImages($fn)
        }
        catch {unset OldImages}
    }
    method _ClearBigImages {} {
        foreach b [array names BigImages] {
            image delete $BigImages($b)
        }
        catch {unset BigImages}
    }
  
    method _ReadFile {name} {
       
        # dgroth fix for files containing anchors
        regexp {(.+)#} $name match name  
        # fix for enabling web browsing    
        if {[string match {http:*} $name]} {
            set t [http::geturl $name];
            set r [http::data $t]
            http::cleanup $t;
            return $r
        }
        if {[catch {open $name r} fp]} {
            tk_messageBox -icon error -message $fp -type ok
            return {}
        } else {
         
            fconfigure $fp -translation binary
            set r [read $fp [file size $name]]
            close $fp
            return $r
        }
    }
    method load {} {
        set filetypes {
            {{Html Files} {.html .htm}}
            {{All Files} *}
        }
        set f [tk_getOpenFile -initialdir $lastDir -filetypes $filetypes]
        if {$f!=""} {
            $self browse $f
            set lastDir [file dirname $f]
        }
    }
}
snit::widget snittabbrowser {
    variable browser
    variable Popupvar
    variable popup
    variable pagename ""
    typevariable px
    option -home ""
    option -fontsize 12
    option -browsecmd 
    option -raisecmd 
    constructor {args} {
        pack [NoteBook $win.nb -side top -width 150 -height 50] -fill both -expand yes -side top
        $win.nb insert end page$px -text " 1 "
        set f [$win.nb getframe page1]
        pack [snitbrowser $f.browser] -side top -expand yes -fill both
        
        set browser(page$px) $f.browser
       
        $win.nb raise [$win.nb page 0]
     
        $win.nb bindtabs <ButtonPress-3> [mymethod _NBPopup]
        $self configurelist $args
        
        $f.browser configure -fontsize $options(-fontsize)
        $f.browser configure -browsecmd $options(-browsecmd)
        $win.nb itemconfigure page1 -raisecmd $options(-raisecmd)
        incr px
    }   
    typeconstructor {
        set px 1
    }
    method pagesget {} {
        # returns a list with pagenames urls
        set liste [list]
        foreach page [$win.nb pages] {
            lappend liste [$win.nb itemcget $page -text]
            lappend liste [$browser($page) geturl]
        }
        return $liste
    }
    method pagesload {liste} {
        set x 1
        set l [llength [$win.nb pages]]
        foreach {pagetext url} $liste {
            if {$x > $l} {
                $self _NBAddPage
            }
            $win.nb itemconfigure page$x -text $pagetext
            $browser(page$x) browse $url
            $browser(page$x) configure -browsecmd $options(-browsecmd)
            incr x
        }
    }
    method _NBDeletePage {page rpage} {
        $win.nb delete $page
        $win.nb raise $rpage
    }
    onconfigure -fontsize {value} {
        set options(-fontsize) $value
        foreach page [$win.nb pages] {
            $browser($page) configure -fontsize $options(-fontsize)
        }
    }
    method _NBAddPage {} {
        $win.nb insert end page$px -text " $px " -raisecmd $options(-raisecmd)
      
        set f [$win.nb getframe page$px]
        pack [snitbrowser $f.browser -fontsize $options(-fontsize)] -side top -expand yes -fill both
        
        set browser(page$px) $f.browser
        $win.nb raise page$px
        if {[winfo exists $Popupvar] } {
            foreach page [$win.nb pages] {
                $browser($page) bind <ButtonPress-3> [mymethod _Popup]
            }
        }
        incr px
            
    }
    method _NBPopup {page} {
        if {[winfo exists .tmenu]} {destroy .tmenu}
        menu .tmenu -tearoff 0
        .tmenu add command -label "New Tab" -command [list $self _NBAddPage]
        set endIndex [llength [$win.nb pages]]
        set curIndex [$win.nb index $page]
        if { $endIndex > 1} {
            set idx $curIndex
            incr idx -1
            set rpage [$win.nb pages $idx]
            .tmenu add command -label "Delete Tab" -command [list $self _NBDeletePage $page $rpage]
            # does not work
        }
        incr endIndex -1
        if { $curIndex > 0} {
            set newIndex $curIndex
            incr newIndex -1
            .tmenu add command -label "switch left" -command [list $win.nb move $page $newIndex]
        }
        
        if { $curIndex < $endIndex } {
            set newIndex $curIndex
            incr newIndex 1
            .tmenu add command -label "switch right" -command [list $win.nb move $page $newIndex]
        }
        .tmenu add separator
        .tmenu add command -label "rename tab" -command [list $self _NBTabRename]
        set x [winfo pointerx .]
        set y [winfo pointery .]
        tk_popup .tmenu $x $y 
        return
    }
    method _NBTabRenameOK {} {
        $win.nb itemconfigure [$win.nb raise] -text $pagename
        destroy .tabrename
    }
    method _NBTabRename {} {
        catch {destroy .tabrename}
        set x [winfo pointerx .]
        set y [winfo pointery .]
        toplevel .tabrename 
        wm geometry .tabrename +$x+$y
        wm title .tabrename "Rename Tab"
        set pagename [$win.nb itemcget [$win.nb raise] -text]
       
        pack [entry .tabrename.entry -textvariable [varname pagename]] -side left -padx 5 -pady 5
        pack [button .tabrename.ok -text OK -width 10 -command [list $self _NBTabRenameOK]] -side left -padx 5 -pady 5
        pack [button .tabrename.cancel -text Cancel -width 10 -command [list destroy .tabrename]] -side left -padx 5 -pady 5
    }
     method _Popup {} {
        if {[winfo exists $Popupvar] } {
            set x [winfo pointerx .]
            set y [winfo pointery .]
            tk_popup $Popupvar $x $y 
        }
    }
    method popup {path} {
        set Popupvar $path
        foreach page [$win.nb pages] {
            $browser($page) bind <ButtonPress-3> [mymethod _Popup]
        }
        menu $Popupvar -tearoff 0
        $Popupvar add command -label Back -underline 1 -accelerator Alt+Left \
                  -command [mymethod back]
        $Popupvar add command -label Forward -underline 1 -accelerator Alt+Right \
                  -command [mymethod forward]
        $Popupvar add command -label Home     -underline 1 -accelerator Alt+Home  \
                  -command [mymethod home]
        $Popupvar add separator  
        $Popupvar add command -label Refresh -underline 0 -accelerator Ctrl-r \
                  -command [mymethod refresh]
        
        
    
        bind all <Alt-Left> [ mymethod back ]
        bind all <Alt-Right> [ mymethod forward ]
        bind all <Alt-Home> [mymethod home]
    }
    method search {text case forward} {
        set raised [$win.nb raise]
        #puts "forward: $forward"
        $browser($raised) search $text $case $forward
    }
    method browse {url} {
        set raised [$win.nb raise]
        $browser($raised) browse $url
    }
    method home {} {
        set raised [$win.nb raise]
        if {$options(-home) ne ""} {
            $browser($raised) browse $options(-home)  
        } else {
            $browser($raised) home
        }
        
    }
    method back {} {
        set raised [$win.nb raise]
        $browser($raised) back
    }
    method forward {} {
         set raised [$win.nb raise]
         $browser($raised) forward
     }
     method refresh {} {
        set raised [$win.nb raise]
        $browser($raised) refresh
    }
    method geturl {} {
        set raised [$win.nb raise]
        $browser($raised) geturl
    }
    method load {} {
        set raised [$win.nb raise]
        $browser($raised) load
    }

}
proc lets_test_it {} {
    global var
    wm title . "Sample Snitbrowser by Dr. Detlef Groth, 2003"
    menu .menubar
    . config -menu .menubar
    set mnu(file) [menu .menubar.file -tearoff 0]
    .menubar add cascade -label File -underline 0 -menu $mnu(file) 
    set mnu(help) [menu .menubar.help -tearoff 0]
    .menubar add cascade -label Help -underline 0 -menu $mnu(help) 
    set var(about) "Sample Snitbrowser by Dr. Detlef Groth"
    append var(about) "\n          2003\n       nocatdetlef@dgroth.de"
    append var(about) "\n      Delete nocat to answer!"
    $mnu(help) add command -label "About ... " -underline 0 -command \
             { tk_messageBox -title "About!" -icon info \
              -message $var(about) -type ok}
    $mnu(file) add command -label Exit -underline 1 -command  {exit 0}
    pack [snittabbrowser .browser -home d:/docs/tcl8.4.4.thinlet/contents.htm] -side left -fill both -expand yes
    .browser browse d:/docs/tcl8.4.4.thinlet/contents.htm
    #.browser browse "http://mini.net/tcl/2993"
    #.browser browse "http://mini.net/tcl/2993"
    # installing the popup
    .browser popup .popup
}

#lets_test_it


